/***********************************************************************************************************************
**
** Copyright (C) 2023 BaseALT Ltd. <org@basealt.ru>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**
***********************************************************************************************************************/
#ifndef LIBDOMAIN_CONNECTION_H
#define LIBDOMAIN_CONNECTION_H

#include <ldap.h>
#include <stdbool.h>
#include <verto.h>

#include "common.h"

#include "request_queue.h"

#define MAX_REQUESTS 8192

enum BindType
{
    BIND_TYPE_INTERACTIVE = 1,          //!< We are going to perform interactive bind.
    BIND_TYPE_SIMPLE      = 2,          //!< We are going to perform simple bind.
};

typedef struct ldap_sasl_options_t
{
    char *mechanism;                   //!< Sasl mechanism to use.

    const char *passwd;                //!< User password.

    bool sasl_nocanon;                 //!< Do not use reverse DNS queries.
    short sasl_flags;                  //!<
    char *sasl_secprops;               //!<
} ldap_sasl_options_t;

typedef struct ldap_sasl_defaults_t
{
    short flags;                      //!<
    char *mechanism;                  //!<

    char *realm;                      //!<
    char *authcid;                    //!<
    char *authzid;                    //!<

    char *passwd;                     //!<
} ldap_sasl_defaults_t;

typedef struct ldap_sasl_params_t
{
    char *dn;                         //!<
    struct berval *passwd;            //!<
    LDAPControl **serverctrls;        //!<
    LDAPControl **clientctrls;        //!<
} ldap_sasl_params_t;

typedef struct ldap_connection_config_t
{
    const char *server;                         //!<
    int port;                                   //!<

    int protocol_verion;                        //!<

    bool chase_referrals;                       //!<
    bool use_start_tls;                         //!<
    bool use_sasl;                              //!<

    int bind_type;                              //!<

    struct ldap_sasl_options_t *sasl_options;   //!<

    int ssl_mode;                               //!<
    const char *tls_ca_cert_file;               //!<
    const char *tls_ca_cert_path;               //!<

    const char *tls_cert_file;                  //!<
    const char *tls_key_file;                   //!<

    bool *tls_require_cert;                     //!<
    int tls_min_protocol_version;               //!<

    int search_timelimit;                       //!<
    int network_timeout;                        //!<
} ldap_connection_config_t;

struct ldap_connection_ctx_t;

typedef struct ld_entry_s ld_entry_t;

typedef enum OperationReturnCode (*operation_callback_fn)(int, LDAPMessage *, struct ldap_connection_ctx_t *);
typedef enum OperationReturnCode (*search_callback_fn)(struct ldap_connection_ctx_t *connection, ld_entry_t** entries);

typedef struct ldhandle LDHandle;

typedef struct ldap_search_request_t
{
    int msgid;                               //!<
    search_callback_fn on_search_operation;  //!<
} ldap_search_request_t;

typedef struct ldap_request_t
{
    int msgid;                                //!<

    operation_callback_fn on_read_operation;  //!<
    operation_callback_fn on_write_operation; //!<

    struct Queue_Node_s node;                 //!<
} ldap_request_t;

typedef struct ldap_connection_ctx_t
{
    LDHandle *handle;                                           //!<

    LDAP *ldap;                                                 //!<

    struct ldap_connection_ctx_t *next;                         //!<
    struct ldap_connection_ctx_t *prev;                         //!<

    int fd;                                                     //!<

    bool handlers_installed;                                    //!<

    struct verto_ctx *base;                                     //!<

    struct verto_ev *read_event;                                //!<
    struct verto_ev *write_event;                               //!<

    operation_callback_fn on_error_operation;                   //!<

    int bind_type;                                              //!<
    int directory_type;                                         //!<
    int msgid;                                                  //!<

    const char *rmech;                                          //!<

    struct request_queue* callqueue;                            //!<

    struct ldap_request_t read_requests[MAX_REQUESTS];          //!<
    struct ldap_request_t write_requests[MAX_REQUESTS];         //!<

    struct ldap_search_request_t search_requests[MAX_REQUESTS]; //!<

    int n_read_requests;                                        //!<
    int n_write_requests;                                       //!<

    int n_search_requests;                                      //!<

    int n_reconnect_attempts;                                   //!<

    struct state_machine_ctx_t *state_machine;                  //!<

    struct ldap_sasl_defaults_t *ldap_defaults;                 //!<
    struct ldap_sasl_params_t *ldap_params;                     //!<

    struct ldap_connection_config_t* config;                    //!<
} ldap_connection_ctx_t;

enum OperationReturnCode connection_configure(struct ldap_global_context_t *global_ctx,
                                              struct ldap_connection_ctx_t *connection,
                                              struct ldap_connection_config_t *config);
enum OperationReturnCode connection_start_tls(struct ldap_connection_ctx_t *connection);
enum OperationReturnCode connection_sasl_bind(struct ldap_connection_ctx_t *connection);
enum OperationReturnCode connection_ldap_bind(struct ldap_connection_ctx_t *connection);
enum OperationReturnCode connection_close(struct ldap_connection_ctx_t *connection);

// Operation handlers.
void connection_on_read(verto_ctx *ctx, verto_ev *ev);
void connection_on_write(verto_ctx *ctx, verto_ev *ev);

enum OperationReturnCode connection_bind_on_read(int, LDAPMessage *, struct ldap_connection_ctx_t *connection);
enum OperationReturnCode connection_start_tls_on_read(int, LDAPMessage *, struct ldap_connection_ctx_t *connection);

#endif //LIBDOMAIN_CONNECTION_H
